package org.bitbucket.bradleysmithllc.etlunit.maven;

import org.bitbucket.bradleysmithllc.etlunit.NullStatusReporterImpl;
import org.bitbucket.bradleysmithllc.etlunit.StatusReporter;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestMethod;
import org.bitbucket.bradleysmithllc.etlunit.util.IOUtils;
import org.bitbucket.bradleysmithllc.etlunit.util.StringUtils;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;

public class SurefireStatusReporter extends NullStatusReporterImpl {
		private String currentTestClass = null;
		private List<TestResult> currentTests = new ArrayList<TestResult>();
		private long testStartTime;
		private long testClassElapsed;
		private int numFailures;
		private int numErrors;
		private final File targetDirectory;

		SurefireStatusReporter(File target)
		{
			targetDirectory = target;
		}

		@Override
		public void testBeginning(ETLTestMethod method) {
			testStartTime = System.currentTimeMillis();

			String qualifiedName = method.getTestClass().getQualifiedName();

			if (currentTestClass != null)
			{
				if (!currentTestClass.equals(qualifiedName))
				{
					publishReport();
					currentTestClass = qualifiedName;
				}
			}
			else
			{
				currentTestClass = qualifiedName;
			}
		}

	private void publishReport() {
		StringBuffer buffer = new StringBuffer("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n");
		buffer.append("<testsuite failures=\"" + numFailures + "\" time=\"" + (testClassElapsed / 1000.0d)  + "\" errors=\"" + numErrors + "\" skipped=\"0\" tests=\"" + currentTests.size() + "\" name=\"" + currentTestClass + "\">\n");
		buffer.append("<properties />\n");

		for (TestResult result : currentTests)
		{
			buffer.append("<testcase time=\"" + (result.elapsedTime / 1000.0d) + "\" classname=\"" + currentTestClass + "\" name=\"" + result.testName + "\">\n");

			if (result.testResult == TestResult.result.error)
			{
				buffer.append("\t<error type=\"" + StringUtils.sanitize(result.failureMessage, '_') + "\"><![CDATA[" + StringUtils.sanitize(result.failureMessage, '_') + "]]></error>\n");
			}
			else if (result.testResult == TestResult.result.fail)
			{
				buffer.append("\t<failure message=\"" + StringUtils.sanitize(result.failureMessage, '_') + "\" type=\"" + StringUtils.sanitize(result.failureMessage, '_') + "\"><![CDATA[" + result.failureMessage + "]]></failure>\n");
			}

			buffer.append("</testcase>\n");
		}

		buffer.append("</testsuite>");

		try {
			IOUtils.writeBufferToFile(new File(targetDirectory, "TEST-" + currentTestClass + ".xml"), buffer);
		} catch (IOException e) {
			throw new RuntimeException("");
		}

		currentTestClass = null;
		numFailures = 0;
		numErrors = 0;
		testClassElapsed = 0L;
		currentTests.clear();
	}

	@Override
	public void testCompleted(ETLTestMethod method, StatusReporter.CompletionStatus status) {
		TestResult result = new TestResult();

		result.testName = method.getName();

		result.elapsedTime = System.currentTimeMillis() - testStartTime;

		testClassElapsed += result.elapsedTime;

		result.testResult = TestResult.result.pass;

		if (status.getAssertionFailures().size() != 0)
		{
			result.testResult = TestResult.result.fail;
			result.failureMessage = status.getAssertionFailures().toString();
			numFailures++;
		}
		else if (status.hasError())
		{
			result.testResult = TestResult.result.error;

			StringWriter stringWriter = new StringWriter();
			PrintWriter writer = new PrintWriter(stringWriter);
			status.getError().printStackTrace(writer);

			result.failureMessage = status.getError().toString() + "\n" + stringWriter.toString();

			numErrors++;
		}

		currentTests.add(result);
	}

	@Override
	public void testsCompleted() {
		if (currentTestClass != null)
		{
			publishReport();
		}
	}
}

final class TestResult {
	enum result {
		error,
		fail,
		pass
	}

	result testResult;
	String testName;
	long elapsedTime;
	String failureMessage;
}